@@ -132,17 +132,28 @@ def calculate_rat_rcwa(structure, size, orders, wavelength, incidence, substrate
     return output
 
 
-def rcwa_rat(S, n_layers):
+def rcwa_rat(S, n_layers, theta, n_inc):
+    """
+    Calculates total reflection and transmission and reflection and transmission by diffraction order, at a single wavelength
+
+    :param S: S4 simulation object
+    :param n_layers: the number of layers in the structure, including the incidence and transmission medium
+    :param theta: incident polar angle in degrees
+    :param n_inc: (real part of) the refractive index of the incidence medium
+    :return: dictionary with total R, total T, and lists of R and T per order
+    """
+    # TODO: check normalization of pfbo values. Also, theta correction should happen inside here rather than externally
     below = 'layer_' + str(n_layers)  # identify which layer is the transmission medium
 
-    R = 1 - sum(S.GetPowerFlux('layer_2'))  # GetPowerFlux gives forward & backward Poynting vector, so sum to get power flux
-    R_pfbo = -np.array(S.GetPowerFluxByOrder('layer_1'))[:,1] # real part of backwards power flow
-    Nrm = np.real(np.sum(R_pfbo))
-    R_pfbo = np.real((R/Nrm)*R_pfbo)
+    R = -n_inc*np.real(S.GetPowerFlux('layer_1')[1])/np.cos(theta*np.pi/180)  # GetPowerFlux gives forward & backward Poynting vector, so sum to get power flux
 
-    T = sum(S.GetPowerFlux(below))
 
-    T_pfbo = np.real(np.sum(np.array(S.GetPowerFluxByOrder(below)), 1))
+    R_pfbo = -np.array(S.GetPowerFluxByOrder('layer_1'))[:,1] # real part of backwards power flow. Not normalised correctly.
+
+    T = n_inc*sum(S.GetPowerFlux(below))/np.cos(theta*np.pi/180)
+
+    T_pfbo = n_inc*np.real(np.sum(np.array(S.GetPowerFluxByOrder(below)), 1))/np.cos(theta*np.pi/180)
+
     return {'R': np.real(R), 'T': np.real(T), 'R_pfbo': R_pfbo, 'T_pfbo': T_pfbo}
 
 
@@ -213,12 +224,21 @@ def initialise_S(size, orders, geom_list, mats_oc, shapes_oc, shape_mats, widths
 
 
 def necessary_materials(geom_list):
+    """Checks which materials are needed to define the shapes in the layers (so not the main layer materials), and
+    returns a version of the geom_list object which doesn't contain Solcore materials (to avoid pickling issues).
+
+    :param geom_list: list of the geometry attributes of every layer in the structure. This is a list of lists;
+            each lists can contain any number of dictionaries (including 0) describing a shape in that layer
+    :return: a list of the necessary Solcore materials for the calculation, and an object which is exactly like
+            geom_list except the Solcore material objects have been replaced with strings. This is to avoid pickling
+            issues when running calculations in parallel.
+    """
     shape_mats = []
     geom_list_str = [None] * len(geom_list)
     for i1, geom in enumerate(geom_list):
         if bool(geom):
             shape_mats.append([x['mat'] for x in geom])
-            geom_list_str[i1] = [{}] * len(geom)
+            geom_list_str[i1] =[{} for _ in range(len(geom))]
             for i2, g in enumerate(geom):
                 for item in g.keys():
                     if item != 'mat':
@@ -230,6 +250,15 @@ def necessary_materials(geom_list):
 
 
 def update_epsilon(S, stack_OS, shape_mats_OS, wl):
+    """Updates the dielectric constants in the S4 simulation object to the relevant values at the desired wavelength.
+
+    :param S: S4 simulation object
+    :param stack_OS: OptiStack for the main layers in the structures
+    :param shape_mats_OS: OptiStack containing layers made up of the materials used in the shapes within the main layers
+            (the structure is not physically meaningful but is used to store the optical constant information)
+    :param wl: wavelength in nm
+    :return: the S4 simulation object with updated parameters
+    """
     for i1 in range(len(stack_OS.get_widths())):
         S.SetMaterial('layer_' + str(i1 + 1), stack_OS.get_indices(wl)[i1] ** 2)
     for i1 in range(len(shape_mats_OS.widths)):  # initialise the materials needed for all the shapes in S4
@@ -250,7 +279,7 @@ def calculate_absorption_profile_rcwa(structure, size, orders, wavelength, rat_o
     :param size: a tuple of 2-D vectors in the format ((ux, uy), (vx, vy)) giving the x and y components of the lattice unit vectors in nm.
     :param orders: number of orders to retain in the RCWA calculations.
     :param wavelength: Wavelengths (in nm) in which calculate the data.
-    :param rat_output: A_pol' (polarization-dependent layer absorption) output from calculate_rat_rcwa
+    :param rat_output: 'A_pol' (polarization-dependent layer absorption) output from calculate_rat_rcwa
     :param z_limit: Maximum value in the z direction at which to calculate depth-dependent absorption (nm)
     :param steps_size: if the dist is not specified, the step size in nm to use in the depth-dependent calculation
     :param dist: the positions (in nm) at which to calculate depth-dependent absorption
@@ -338,22 +367,40 @@ def calculate_absorption_profile_rcwa(structure, size, orders, wavelength, rat_o
     return output
 
 
-def rcwa_position_resolved(S, layer, depth, A):
+def rcwa_position_resolved(S, layer, depth, A, theta, n_inc):
+    """Calculate the derivative of the absorbed power with respect to depth in a layer.
+
+    :param S: S4 simulation object
+    :param layer: layer index (layer 1 is the incidence medium)
+    :param depth: depth in the layer in nm
+    :param A: the total power absorbed in the layer
+    :param theta: polar angle of the light in the incidence medium
+    :param n_inc: (real part of) the refractive index of the incidence medium
+    :return: the derivative of the absorbed power with depth in the structure
+    """
     if A > 0:
         delta = 1e-9
         power_difference = np.real(
             sum(S.GetPowerFlux(layer, depth - delta)) - sum(S.GetPowerFlux(layer, depth + delta)))
-        return power_difference / (2 * delta)  # absorbed energy density normalised to total absorption
+        return n_inc * power_difference / (2 * delta * np.cos(theta*np.pi/180))  # absorbed energy density normalised to total absorption
     else:
         return 0
 
-def rcwa_absorption_per_layer(S, n_layers):
-    # layer 1 is incidence medium, layer n is the transmission medium
+def rcwa_absorption_per_layer(S, n_layers, theta, n_inc):
+    """Calculates the fraction of the incident power absorbed in each layer of the structure.
+
+    :param S: S4 simulation object
+    :param n_layers: number of layers in the structure, including the incidence and transmission medium
+    :param theta: polar angle of the light in the incidence medium
+    :param n_inc: (real part of) the refractive index of the incidence medium
+    :return:
+    """
     A = np.empty(n_layers-2)
     for i1, layer in enumerate(np.arange(n_layers-2)+2):
-        A[i1] = np.real(sum(S.GetPowerFlux('layer_' + str(layer))) - sum(S.GetPowerFlux('layer_' + str(layer+1))))
+        A[i1] = np.real(sum(S.GetPowerFlux('layer_' + str(layer))) -
+                        sum(S.GetPowerFlux('layer_' + str(layer+1))))
 
-    A = np.array([x if x > 0 else 0 for x in A])
+    A = n_inc*np.array([x if x > 0 else 0 for x in A])/np.cos(theta*np.pi/180)
 
     return A
 
@@ -382,10 +429,10 @@ def RCWA_wl(wl, geom_list, l_oc, s_oc, s_names, pol, theta, phi, widths, size, o
     def vs_pol(s, p):
         S.SetExcitationPlanewave((theta, phi), s, p, 0)
         S.SetFrequency(1 / wl)
-        out = rcwa_rat(S, len(widths))
+        out = rcwa_rat(S, len(widths), theta, n_inc)
         R = out['R']
         T = out['T']
-        A_layer = rcwa_absorption_per_layer(S, len(widths))
+        A_layer = rcwa_absorption_per_layer(S, len(widths), theta, n_inc)
         return R, T, A_layer
 
     S = initialise_S(size, orders, geom_list, l_oc, s_oc, s_names, widths, rcwa_options)
@@ -407,8 +454,6 @@ def RCWA_wl(wl, geom_list, l_oc, s_oc, s_names, pol, theta, phi, widths, size, o
             T = (T_s + T_p) / 2
             A_layer = np.stack([A_layer_s, A_layer_p])
 
-    T = n_inc * T / np.cos(theta * np.pi / 180)
-    A_layer = n_inc * A_layer / np.cos(theta * np.pi / 180)
     return R, T, A_layer
 
 
@@ -440,7 +485,7 @@ def RCWA_wl_prof(wl, A, dist, geom_list, layers_oc, shapes_oc, s_names, pol, the
         for j, d in enumerate(dist):
             layer, d_in_layer = tmm.find_in_structure_with_inf(widths, d)
             layer_name = 'layer_' + str(layer + 1)  # layer_1 is air above so need to add 1
-            data = rcwa_position_resolved(S, layer_name, d_in_layer, A_total)
+            data = rcwa_position_resolved(S, layer_name, d_in_layer, A_total, theta, n_inc)
             profile[j] = data
 
         return profile
@@ -460,5 +505,5 @@ def RCWA_wl_prof(wl, A, dist, geom_list, layers_oc, shapes_oc, s_names, pol, the
             profile_p = vs_pol_prof(0, 1, A[1])
             profile_data = 0.5*(profile_s + profile_p)
 
-    profile_data = n_inc * profile_data / np.cos(theta * np.pi / 180)
+    profile_data = profile_data
     return profile_data
