@@ -1,6 +1,6 @@
 from .IV import iv_multijunction
 from .QE import spectral_response_all_junctions
 from .detailed_balance import iv_detailed_balance, qe_detailed_balance, absorptance_detailed_balance, db_options
-from .depletion_approximation import iv_depletion, qe_depletion
+from .depletion_approximation import iv_depletion, qe_depletion, da_options
 from .diode_equation import iv_2diode
 from .tunnel_junctions import resistive_tunnel_junction, parametric_tunnel_junction, external_tunnel_junction
\ No newline at end of file
