@@ -1,8 +1,11 @@
-from pytest import approx
+from pytest import approx, mark
 
+
+@mark.flaky(reruns=5)
 def test_DE():
     from yabox.problems import Ackley
     from solcore.optimization import DE
+
     problem = Ackley()
 
     algorithm = DE(problem, problem.bounds)
@@ -16,12 +19,14 @@ def test_DE():
     assert result[4][-1] == approx(0.0)
 
 
+@mark.flaky(reruns=5)
 def test_PDE():
     from yabox.problems import Ackley
     from solcore.optimization import PDE
+
     problem = Ackley()
 
-    algorithm = PDE(problem, problem.bounds)
+    algorithm = PDE(problem, problem.bounds, processes=0)
 
     result = algorithm.solve()
 
